<?php
namespace AppZap\PHPFramework\Mvc;

class Request {

  /**
   * @var array
   */
  protected $parameters;

  /**
   * @var string
   */
  protected $request_method;

  /**
   * @var array
   */
  protected $value_sources = [
    'get' => ['default' => 'get', 'fallback' => 'post'],
    'post' => ['default' => 'post', 'fallback' => 'get'],
    'head' => ['default' => 'get', 'fallback' => 'post'],
    'cli' => ['default' => 'cli'],
  ];

  /**
   * @param string $request_method
   * @param array $parameters
   *
   * @throws \AppZap\PHPFramework\Mvc\MethodNotSupportedException
   */
  public function __construct($request_method, $parameters = []) {
    // Early exit when not defined where to read request values
    if(!array_key_exists($request_method, $this->value_sources)) {
      throw new MethodNotSupportedException('Getting request parameters of ' . $request_method . ' is not supported.', 1415273543);
    }
    $this->request_method = $request_method;
    $this->parameters = $parameters;
  }

  /**
   * Returns a parameter value from the request sent by the client
   *
   * @param string $parameter_name Name of the request parameter to deliver
   * @param mixed $default_value Default value to deliver when parameter is not found
   * @param boolean $use_strict_mode Whether to allow searching in other request types than the used one
   * @return mixed
   * @throws MethodNotSupportedException when the parameter is read for a request type which is not supported by the framework
   */
  public function get($parameter_name, $default_value = NULL, $use_strict_mode = FALSE) {

    try {
      return $this->read_request_parameter($parameter_name, $this->value_sources[$this->request_method]['default']);
    } catch(ParameterNotFoundException $ex) {}

    if(!$use_strict_mode && isset($this->value_sources[$this->request_method]['fallback'])) {
      try {
        return $this->read_request_parameter($parameter_name, $this->value_sources[$this->request_method]['fallback']);
      } catch(ParameterNotFoundException $ex) {}
    }

    return $default_value;
  }

  /**
   * @return array
   */
  public function getParameters() {
    return $this->parameters;
  }

  /**
   * @return string
   */
  public function getRequestMethod() {
    return $this->request_method;
  }

  /**
   * Reads the plain request body as a string
   *
   * @return string
   * @throws ParameterNotFoundException if no body could be read
   */
  public function body() {
    $body = '';
    $fh   = @fopen('php://input', 'r');
    if ($fh) {
      while (!feof($fh)) {
        $s = fread($fh, 1024);
        if (is_string($s)) {
          $body .= $s;
        }
      }
      fclose($fh);
    } else {
      throw new ParameterNotFoundException('No body could be read.');
    }

    return $body;
  }

  /**
   * Returns the value of an parameter for a specified source if the source is supported and the parameter exists
   *
   * @param string $parameter_name Name of the request parameter to deliver
   * @param string $value_source Name of the source to use when reading the parameter
   * @return mixed
   * @throws ValueSourceNotSupportedException when the name of the source is unsupported
   * @throws ParameterNotFoundException when the parameter is not found in the source to avoid collision with other value types
   */
  protected function read_request_parameter($parameter_name, $value_source) {
    $source = null;
    if($value_source == 'get') {
      $source = &$_GET;
    } elseif($value_source == 'post') {
      $source = &$_POST;
    }

    if($source === null) {
      throw new ValueSourceNotSupportedException('The source ' . $value_source . ' is not supported to read values from.', 1415273682);
    }

    if(array_key_exists($parameter_name, $source)) {
      return $source[$parameter_name];
    }

    throw new ParameterNotFoundException('Parameter ' . $parameter_name . ' was not found in source ' . $value_source, 1415273689);
  }

}
